import base64
import importlib
import json
import os

from django.apps import apps
from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponseRedirect, HttpResponse, JsonResponse
from django.utils.cache import add_never_cache_headers
try:
    from django.utils.encoding import force_text
except ImportError:
    from django.utils.encoding import force_str as force_text  # type: ignore
from django.utils.functional import Promise
from django.views.generic import View
from django.views.generic.base import TemplateView


class LazyEncoder(DjangoJSONEncoder):
    """Encodes django's lazy i18n strings
    """
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)


# =================================
# Mixins
# =================================

class WidgetInitKwargsMixin:

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.extra_data = {
            "args": args,
            "kwargs": kwargs
        }

    def get_encoded_extra_data(self):
        data_json = json.dumps(self.extra_data)
        payload_bytes = base64.b64encode(data_json.encode())
        payload = payload_bytes.decode()
        return payload

    def get_decoded_extra_data(self, request):
        encoded_extra_data = request.GET.get("extra_data", None)
        if not encoded_extra_data:
            return {}
        extra_bytes = encoded_extra_data.encode()
        payload_bytes = base64.b64decode(extra_bytes)
        payload = json.loads(payload_bytes.decode())
        return payload

    def convert_extra(self, extra_item, first=True):
        if first:
            start = '?'
        else:
            start = '&'
        if type(extra_item) == dict:
            extra_list = []
            for k, v in extra_item.items():
                extra_list.append(f"{k}={v}")
            extra = f"{start}{'&'.join(extra_list)}"
            return extra
        return ''


class JSONResponseMixin:
    is_clean = False

    def render_to_response(self, context):
        """ Returns a JSON response containing 'context' as payload
        """
        return self.get_json_response(context)

    def get_json_response(self, content, **httpresponse_kwargs):
        """ Construct an `HttpResponse` object.
        """
        response = HttpResponse(content,
                                content_type='application/json',
                                **httpresponse_kwargs)
        add_never_cache_headers(response)
        return response

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        self.request = request
        self.csrf_token = self.request.GET.get('csrf_token', None)
        response = None

        func_val = self.get_context_data(**kwargs)
        if not self.is_clean:
            assert isinstance(func_val, dict)
            response = dict(func_val)
            if 'error' not in response and 'sError' not in response:
                response['result'] = 'ok'
            else:
                response['result'] = 'error'
        else:
            response = func_val
        # can't have 'view' here, because the view object can't be jsonified
        junk = response.pop('view', None)

        dump = json.dumps(response, cls=LazyEncoder)
        return self.render_to_response(dump)


# =================================
# Views
# =================================


class JSONResponseView(JSONResponseMixin, TemplateView):
    pass


class JSONDataView(View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data()
        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        return {}

    def render_to_response(self, context, **response_kwargs):
        return JsonResponse(context)


class WildewidgetDispatch(WidgetInitKwargsMixin, View):

    def dispatch(self, request, *args, **kwargs):
        # initkwargs = {}

        wildewidgetclass = request.GET.get('wildewidgetclass', None)
        csrf_token = request.GET.get('csrf_token', '')
        if wildewidgetclass:
            configs = apps.get_app_configs()
            for config in configs:
                check_file = os.path.join(config.path, "wildewidgets.py")
                check_dir = os.path.join(config.path, "wildewidgets")
                if os.path.isfile(check_file) or os.path.isdir(check_dir):
                    module = importlib.import_module(f"{config.name}.wildewidgets")
                    if hasattr(module, wildewidgetclass):
                        class_ = getattr(module, wildewidgetclass)
                        extra_data = self.get_decoded_extra_data(request)
                        initargs = extra_data.get('args', [])
                        initkwargs = extra_data.get('kwargs', {})
                        instance = class_(*initargs, **initkwargs)
                        instance.request = request
                        instance.csrf_token = csrf_token
                        instance.args = initargs
                        instance.kwargs = initkwargs
                        return instance.dispatch(request, *args, **kwargs)


class TableActionFormView(View):
    http_method_names = ['post']
    url = None

    def process_form_action(self, action, items):
        pass

    def post(self, request, *args, **kwargs):
        checkboxes = request.POST.getlist('checkbox')
        action = request.POST.get('action')
        self.process_form_action(action, checkboxes)
        return HttpResponseRedirect(self.url)


class TableView(TemplateView):
    table_class = None

    def get_context_data(self, **kwargs):
        if not self.table_class:
            raise ImproperlyConfigured(
                "You must set a table_class attribute on {}".format(self.__class__.__name__)
            )
        kwargs['table'] = self.table_class()
        return super().get_context_data(**kwargs)


class StandardWidgetMixin():

    def get_context_data(self, **kwargs):
        kwargs['content'] = self.get_content()
        breadcrumbs = self.get_breadcrumbs()
        kwargs['breadcrumbs'] = breadcrumbs
        if breadcrumbs:
            kwargs['page_title'] = breadcrumbs.flatten()
        return super().get_context_data(**kwargs)

    def get_content(self):
        raise ImproperlyConfigured(
                "You must override get_content in {}".format(self.__class__.__name__)
            )
        return None

    def get_breadcrumbs(self):
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from copy import copy
import random
from urllib.parse import urlencode

from django.core.exceptions import ImproperlyConfigured
from django.core.paginator import InvalidPage, Paginator
from django.db.models import QuerySet
from django.http import Http404

from .base import TemplateWidget, Block
from .text import HTMLWidget, StringBlock
from .buttons import FormButton
from .headers import CardHeader

from django.core.exceptions import ImproperlyConfigured


class TabbedWidget(TemplateWidget):
    """Extends TemplateWidget.
    Render a tabbed interace. 
    
    Example::

        tab = TabbedWidget()
        tab.add_tab('My First Widget', widget1)
        tab.add_tab('My Second Widget', widget2)
    """
    template_name = 'wildewidgets/tabbed_widget.html'
    slug_suffix = None

    def __init__(self, *args, **kwargs):
        self.tabs = []

    def add_tab(self, name, widget):
        """
        Add a Bootstrap 5 tab named `name` that displays `widget`.

        Args:
            name (str): Tab name/title.
            widget (obj): Widget to display in tab.
        """
        self.tabs.append((name, widget))

    def get_context_data(self, **kwargs):
        kwargs['tabs'] = self.tabs
        if not self.slug_suffix:
            self.slug_suffix = random.randrange(0, 10000)
        kwargs['identifier'] = self.slug_suffix
        return kwargs


class CardWidget(Block):
    """Extends TemplateWidget.
    Renders a Bootstrap 5 Card.

    Args:
        header (header widget, optional): A header widget to display above the card.
        header_text (str, optional): Text used to build a header widget is one is not provided
        card_title (str): Card title
        card_subtitle (str): Card subtitle
        widget (obj): Widget to display in card body
        widget_css (str): CSS to apply to the widget displayed in the card body
        css_class (str): CSS to apply to the card itself in addition to the defaults
    """
    template_name = 'wildewidgets/card_block.html'
    header = None
    header_text = None
    card_title = None
    card_subtitle = None
    widget = None
    widget_css = None

    def __init__(self, *args, **kwargs):
        self.header = kwargs.pop('header', self.header)
        self.header_text = kwargs.pop('header_text', self.header_text)
        if self.header_text and not self.header:
            self.header = CardHeader(header_text=self.header_text)
        self.card_title = kwargs.pop('card_title', self.card_title)
        self.card_subtitle = kwargs.pop('card_subtitle', self.card_subtitle)
        self.widget = kwargs.pop('widget', self.widget)
        self.widget_css = kwargs.pop('widget_css', self.widget_css)
        css_class = kwargs.get('css_class', "")
        if css_class:
            kwargs['css_class'] = f"card {css_class}"
        else:
            kwargs['css_class'] = "card"
        super().__init__(*args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs = super().get_context_data(**kwargs)
        kwargs['header'] = self.header
        kwargs['header_text'] = self.header_text
        kwargs['title'] = self.card_title
        kwargs['subtitle'] = self.card_subtitle
        kwargs['widget'] = self.widget
        kwargs['widget_css'] = self.widget_css
        kwargs['css_class'] = self.css_class

        if not self.widget:
            raise ImproperlyConfigured("You must define a widget.")
        return kwargs

    def set_widget(self, widget, css_class=None):
        """
        Set the widget displayed in the card body

        Args:
            widget (obj): Widget to display in card body
            css_class (str): CSS to apply to the widget displayed in the card body
        """
        self.widget = widget
        self.widget_css = css_class

    def set_header(self, header):
        """
        Set the header widget to display above the card

        Args:
            header (obj): Header widget to display above the card
        """
        self.header = header


class MultipleModelWidget(Block):
    """Extends Block.
    Base class for `PagedModelWidget` and `ListModelWidget`.
    """
    model = None
    model_widget = None
    model_kwargs = {}
    ordering = None
    queryset = None
    item_label="item"

    def __init__(self, *args, item_label="", **kwargs):
        self.model = kwargs.pop('model', self.model)
        self.model_widget = kwargs.pop('model_widget', self.model_widget)
        self.ordering = kwargs.pop('ordering', self.ordering)
        self.queryset = kwargs.pop('queryset', self.queryset)
        self.model_kwargs = kwargs.pop('model_kwargs', copy(self.model_kwargs))
        self.item_label = item_label if item_label else self.item_label
        super().__init__(*args, **kwargs)

    def get_item_label(self, object):
        """
        Returns the type of the item displayed. This is used in the confirmation dialog when deleting.
        """
        return self.item_label

    def get_model_widget(self, object=object, **kwargs):
        """
        Returns the individual widget to display as part of the list.
        """
        if self.model_widget:
            return self.model_widget(object=object, **kwargs)
        else:
            raise ImproperlyConfigured(
                "%(cls)s is missing a model widget. Define "
                "%(cls)s.model_widget or override "
                "%(cls)s.get_model_widget()." % {'cls': self.__class__.__name__}
            )

    def get_model_widgets(self, object_list):
        widgets = []
        for object in object_list:
            widgets.append(self.get_model_widget(object=object, **self.model_kwargs))
        return widgets

    def get_queryset(self):
        """
        Return the list of items for this widget.
        The return value must be an iterable and may be an instance of
        `QuerySet` in which case `QuerySet` specific behavior will be enabled.
        """
        if self.queryset is not None:
            queryset = self.queryset
            if isinstance(queryset, QuerySet):
                queryset = queryset.all()
        elif self.model is not None:
            queryset = self.model._default_manager.all()
        else:
            raise ImproperlyConfigured(
                "%(cls)s is missing a QuerySet. Define "
                "%(cls)s.model, %(cls)s.queryset, or override "
                "%(cls)s.get_queryset()." % {
                    'cls': self.__class__.__name__
                }
            )
        ordering = self.ordering
        if ordering:
            if isinstance(ordering, str):
                ordering = (ordering,)
            queryset = queryset.order_by(*ordering)
        return queryset


class PagedModelWidget(MultipleModelWidget):
    """
    Extends `MultipleModelWidget`. A widget that displays a pageable list of widgets 
    defined by a queryset. A model or queryset must be provided, and either a model_widget 
    or the funtion `get_model_widget` must be provided.

    Args:
        page_kwarg (str, optional): get argument for the page number. Defaults to `page`.
        paginate_by (int, optional): number of widgets per page. Defaults to all widgets.
        extra_url (dict, optional): extra paramters passed in the page links.

    Example::

        PagedModelWidget(
            queryset=project.myobject_set.all(),
            paginate_by=3,
            page_kwarg='myobject_page',
            model_widget=MyObjectWidget,
            model_kwargs={'foo': bar},
            item_label="myobject",
            extra_url={
                'pk': myobject.id,
                '#':'myobjects',
            },
        )
    """
    template_name = 'wildewidgets/paged_model_widget.html'
    page_kwarg = 'page'
    paginate_by = None
    max_page_controls = 5
    css_class = "wildewidgets-paged-model-widget "

    def __init__(self, *args, **kwargs):
        self.page_kwarg = kwargs.pop('page_kwarg', self.page_kwarg)
        self.paginate_by = kwargs.pop('paginate_by', self.paginate_by)
        self.extra_url = kwargs.pop('extra_url', {})
        super().__init__(*args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs = super().get_context_data(**kwargs)
        self.request = kwargs.get('request')
        if self.paginate_by:
            paginator = Paginator(self.get_queryset(), self.paginate_by)
            page_number = self.request.GET.get(self.page_kwarg)
            try:
                page_number = int(page_number)
            except (TypeError, ValueError):
                page_number = 1
            try:
                page = paginator.page(page_number)
                kwargs['widget_list'] = self.get_model_widgets(page.object_list)
                kwargs['page_obj'] = page
                kwargs['is_paginated'] = page.has_other_pages()
                kwargs['paginator'] = paginator
                kwargs['page_kwarg'] = self.page_kwarg
                pages = kwargs['page_obj'].paginator.num_pages
                if pages > self.max_page_controls:
                    pages = self.max_page_controls
                page_number = page.number
                max_controls_half = int(self.max_page_controls / 2)
                range_start = 1 if page_number - max_controls_half < 1 else page_number - max_controls_half
                kwargs['page_range'] = range(range_start, range_start+pages)
            except InvalidPage as e:
                raise Http404(
                    'Invalid page (%(page_number)s): %(message)s' % {
                        'page_number': page_number,
                        'message': str(e)
                    }
                )
        else:
            kwargs['widget_list'] = self.get_model_widgets(self.get_queryset().all())
        kwargs['item_label'] = self.item_label
        if self.extra_url:
            anchor = self.extra_url.pop("#", None)
            extra_url = f"&{urlencode(self.extra_url)}"
            if anchor:
                extra_url = f"{extra_url}#{anchor}"
            kwargs['extra_url'] = extra_url
        else:
            kwargs['extra_url'] = ''
        return kwargs


class CrispyFormWidget(Block):
    """Extends Block.
    A widget that displays a crispy_form widget.

    Args:
        form (obj, optional): A crispy form to display. If none is specified, 
            it will be assumed that `form` is specified elsewhere in the context.
    """
    template_name = 'wildewidgets/crispy_form_widget.html'
    css_class = "wildewidgets-crispy-form-widget"

    def __init__(self, *args, form=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.form = form

    def get_context_data(self, **kwargs):
        kwargs = super().get_context_data(**kwargs)
        if self.form:
            kwargs['form'] = self.form
        return kwargs


class CollapseWidget(Block):
    """Extends Block.
    A widget that is initially displayed hidden. It is typically used in conjunction
    with a `CollapseButton`. Pressing the `CollapseButton` toggles the visibility of
    the widget.

    Args:
        css_id (str): This ID will be shared with the `CollapseButton`.

    Example::

        CollapseWidget(
            CrispyFormWidget(
                form=form,
            ),
            css_id=collapse_id,
            css_class="pt-3",
        )
        header = CardHeader(header_text="Services")
        header.add_collapse_button(
            text="Manage",
            color="primary",
            target=f"#{collapse_id}",
        )
    """
    css_class="collapse"

    def __init__(self, *args, **kwargs):
        if "css_class" in kwargs:
            kwargs['css_class'] = f"{kwargs['css_class']} collapse"
        else:
            kwargs['css_class'] = self.css_class
        super().__init__(*args, **kwargs)


class HorizontalLayoutBlock(Block):
    """Extends Block.
    A container widget intended to display several other widgets aligned horizontally.

    Args:
        align (str, optional): Flex alignment as defined in Bootstrap 5 (start, end, center...). 
            Default is `center`.
        justify (str, optional): Flex justification as defined in Bootstrap 5 (start, end, center, between...).
            Default is `between`.
        flex_size (str, optional): Flex size as defined in Bootstrap 5 - sm, md, lg, xl, xxl. Default is None.

    Example::

        HorizontalLayoutBlock(
            LabelBlock(label),
            block1,
            block2,
            justify="end",
            css_class="mt-3",
        )
    """
    align="center"
    justify="between"
    flex_size=None

    def __init__(self, *blocks, **kwargs):
        align = kwargs.pop("align", self.align)
        justify = kwargs.pop("justify", self.justify)
        flex_size = kwargs.pop("flex_size", self.flex_size)
        if flex_size:
            flex = f"d-{flex_size}-flex"
        else:
            flex = "d-flex"
        css_class = kwargs.get("css_class", "")
        css_class += f" {flex} align-items-{align} justify-content-{justify}"
        kwargs["css_class"] = css_class
        super().__init__(*blocks, **kwargs)


class ListModelWidget(MultipleModelWidget):
    """
    Extends `MultipleModelWidget`. This class provides a list of objects
    defined by a QuerySet, displayed in a list-group `ul` block. By default,
    a widget will be provided that simply displays whatever returns from
    the conversion of the object to a `str`. If a `remove_url` is provided,
    an `X` icon to the right of each object will act as a button to remove
    the item.

    Args:
        remove_url (str, optional): The url to `POST` to in order to delete or 
            remove the object. 

    Example::

        widget = ListModelWidget(
            queryset=myparent.children,
            item_label='child',
            remove_url=reverse('remove_url') + "?id={}",
        )
    """
    base_css_class = "wildewidgets-list-model-widget list-group"
    tag='ul'
    remove_url=None

    def __init__(self, *args, remove_url=None, **kwargs):
        self.remove_url = remove_url if remove_url else self.remove_url
        css_class = kwargs.get("css_class", "")
        css_class += f" {self.base_css_class}"
        kwargs["css_class"] = css_class.strip()
        super().__init__(*args, **kwargs)
        self.remove_url = remove_url
        super().__init__(*args, **kwargs)
        widgets = self.get_model_widgets(self.get_queryset().all())
        if not widgets:
            self.add_block(StringBlock(f"No {self.item_label}s", tag='li', css_class="list-group-item fw-light fst-italic"))
        for widget in widgets:
            self.add_block(widget)

    def get_remove_url(self, object):
        if self.remove_url:
            return self.remove_url.format(object.id)
        return ""

    def get_confirm_text(self, object):
        item_label = self.get_item_label(object)
        return f"Are you sure you want to remove this {item_label}?"

    def get_object_text(self, object):
        return str(object)

    def get_model_subblock(self, object):
        if hasattr(object, 'get_absolute_url'):
            url = object.get_absolute_url()
            return HTMLWidget(html=f'<a href="{url}">{self.get_object_text(object)}</a>')
        else:
            return StringBlock(self.get_object_text(object))

    def get_model_widget(self, object=object, **kwargs):
        if self.model_widget:
            return super().get_model_widget(object=object, **kwargs)
        widget = HorizontalLayoutBlock(
            tag='li',
            css_class='list-group-item'
        )
        widget.add_block(self.get_model_subblock(object))
        remove_url = self.get_remove_url(object)
        if remove_url:
            widget.add_block(
                FormButton(
                    close=True,
                    action=remove_url,
                    confirm_text=self.get_confirm_text(object),
                ),
            )
        return widget




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random

from django import template

from wildewidgets.views import JSONDataView

from ..base import Widget


class AltairChart(Widget, JSONDataView):
    template_file = 'wildewidgets/altairchart.html'
    title = None
    width = "100%"
    height = "300px"

    def __init__(self, *args, **kwargs):
        self.data = None
        self.options = {
            'width': kwargs.get('width', self.width),
            'height': kwargs.get('height', self.height),
            "title": kwargs.get('title', self.title)
        }

    def get_content(self, **kwargs):
        chart_id = random.randrange(0, 1000)
        template_file = self.template_file
        if self.data:
            context = self.get_context_data()
        else:
            context = {"async": True}
        html_template = template.loader.get_template(template_file)
        context['options'] = self.options
        context['name'] = f"altair_chart_{chart_id}"
        context["wildewidgetclass"] = self.__class__.__name__
        content = html_template.render(context)
        return content

    def __str__(self):
        return self.get_content()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        self.load()
        context.update({"data": self.data})
        return context

    def set_data(self, spec, set_size=True):
        if set_size:
            self.data = spec.properties(
                width="container",
                height="container"
            ).to_dict()
        else:
            self.data = spec.to_dict()

    def load(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional, Dict, Any

from wildewidgets.widgets.text import StringBlock

from .base import Block, TemplateWidget
from .structure import HorizontalLayoutBlock
from .text import (
    CodeWidget,
    StringBlock,
)

@dataclass
class BreadcrumbItem:
    title: str
    url: str


class BreadrumbBlock(Block):
    template_name: str = 'wildewidgets/breadcrumb_block.html'
    tag: str = 'nav'
    aria_attributes: Dict[str, str] = {'label':'breadcrumb'}
    title_class: str = ""

    def __init__(self, *args, **kwargs):
        self.title_class = kwargs.pop('title_class', self.title_class)
        super().__init__(*args, **kwargs)
        self.items = []

    def add_breadcrumb(self, title, url=None):
        self.items.append(BreadcrumbItem(title, url))

    def get_context_data(self, **kwargs) -> Dict[str, Any]:
        kwargs = super().get_context_data(**kwargs)
        kwargs['items'] = self.items
        kwargs['title_class'] = self.title_class        
        return kwargs

    def flatten(self):
        return ' - '.join([item.title for item in self.items])


class KeyValueListBlock(Block):
    tag = 'ul'
    css_class = "list-group"

    def __init__(self, *args, **kwargs):
        if "css_class" in kwargs:
            kwargs['css_class'] = f"{kwargs['css_class']} {self.css_class}"
        else:
            kwargs['css_class'] = self.css_class
        super().__init__(*args, **kwargs)

    def add_simple_key_value(self, key, value):
        self.add_block(
            HorizontalLayoutBlock(
                StringBlock(key),
                StringBlock(value),
                tag = "li",
                css_class = "list-group-item",
            )
        )

    def add_code_key_value(self, key, value, language=None):
        self.add_block(
            Block(
                StringBlock(key),
                CodeWidget(
                    code=value, 
                    language=language,
                    css_class="m-3",
                ),
                tag = "li",
                css_class = "list-group-item",
            )
        )


class GravatarWidget(TemplateWidget):
    template_name = 'wildewidgets/gravatar.html'
    css_class = "rounded-circle"
    size = "28"

    def __init__(self, *args, gravatar_url=None, size=None, fullname=None, **kwargs):
        self.gravatar_url = gravatar_url
        self.css_class = kwargs.pop('css_class', self.css_class)
        self.size = size or self.size
        self.fullname = fullname
        super().__init__(*args, **kwargs)

    def get_context_data(self, **kwargs) -> Dict[str, Any]:
        kwargs = super().get_context_data(**kwargs)
        kwargs['gravatar_url'] = self.gravatar_url
        kwargs['css_class'] = self.css_class
        kwargs['size'] = self.size
        kwargs['fullname'] = self.fullname
        return kwargs


class InitialsAvatarWidget(TemplateWidget):
    template_name = 'wildewidgets/initials_avatar.html'
    size = 28
    color = "white"
    background_color = "#626976"

    def __init__(self, *args, initials=None, size=None, color=None, background_color=None, fullname=None, **kwargs):
        self.initials = initials
        self.size = size or self.size
        self.color = color or self.color
        self.background_color = background_color or self.background_color
        self.fullname = fullname
        super().__init__(*args, **kwargs)

    def get_context_data(self, **kwargs) -> Dict[str, Any]:
        kwargs = super().get_context_data(**kwargs)
        kwargs['initials'] = self.initials.upper()
        kwargs['fullsize'] = str(self.size)
        kwargs['halfsize'] = str(self.size/2)
        kwargs['color'] = self.color
        kwargs['background_color'] = self.background_color
        kwargs['fullname'] = self.fullname
        return kwargs


class HTMLList(Block):
    template_name: str = 'wildewidgets/html_list.html'
    tag: str = 'ul'

    def __init__(self, *args, items: list = [], **kwargs):
        super().__init__(*args, **kwargs)
        self.items = items

    def add_item(self, item):
        self.items.append(item)

    def get_context_data(self, **kwargs) -> Dict[str, Any]:
        kwargs = super().get_context_data(**kwargs)
        kwargs['items'] = self.items
        return kwargs


class FontIcon(Block):

    tag = 'i'
    prefix: str = 'bi'
    color: Optional[str] = None
    background: Optional[str] = None

    def __init__(
        self,
        icon: str,
        color: Optional[str] = None,
        background: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(**kwargs)
        self.color = color or self.color
        self.background = background or self.background
        self.icon = f'{self.prefix}-{icon}'
        if self._css_class is None:
            self._css_class = ''
        self._css_class += f' {self.icon}'
        if self.color:
            self._css_class += f' text-{self.color} bg-transparent'
        elif self.background:
            self._css_class += f' bg-{self.background} text-{self.background}-fg'


class TablerFontIcon(FontIcon):
    """
    :py:class:`FontIcon` for Tabler Icons.

    Requires::
    
        <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@tabler/icons@latest/iconfont/tabler-icons.min.css">
    """
    prefix = 'ti ti'
